package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.ExportDiagnoseDTO;
import com.diagbot.entity.KlDiagnoseDetail;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.KlDiagnoseDetailServiceImpl;
import com.diagbot.util.*;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 诊断依据明细表 服务实现类
 * </p>
 *
 * @author zhoutg
 * @since 2021-06-10
 */
@Service
public class KlDiagnoseDetailFacade extends KlDiagnoseDetailServiceImpl {

    @Autowired
    KlDiagnoseDetailFacade klDiagnoseDetailFacade;

    /**
     * 导出诊断依据【初版】
     */
    public void exportDiagnose() {
        // 获取全部数据、分组
        List<KlDiagnoseDetail> klDiagnoseDetailList = klDiagnoseDetailFacade.list(new QueryWrapper<KlDiagnoseDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
        Map<String, List<KlDiagnoseDetail>> diagnoseDetailMap = EntityUtil.makeEntityListMap(klDiagnoseDetailList, "disName");
        // TODO 获取公式，分隔编码，替换编码，组装导出数据  Map<String, List<ExportDiagnoseDTO>>
        Map<String, List<ExportDiagnoseDTO>> diagnoseDetails = processDiagnoseDetail(diagnoseDetailMap);


        // TODO 导出数据
    }


    public Map<String, List<ExportDiagnoseDTO>> processDiagnoseDetail(Map<String, List<KlDiagnoseDetail>> diagnoseDetailMap){
        Map<String, List<ExportDiagnoseDTO>> data = new LinkedHashMap<>();
        for(Map.Entry<String, List<KlDiagnoseDetail>> kd : diagnoseDetailMap.entrySet()){
            String sheetName = kd.getKey();
            List<KlDiagnoseDetail> datats = kd.getValue();
            removeKlDiagnoseDetail(datats);
            List<ExportDiagnoseDTO> eds = Lists.newArrayList();
            for (KlDiagnoseDetail kdl:datats) {
                ExportDiagnoseDTO exportDiagnoseDTO = new ExportDiagnoseDTO();
                BeanUtil.copyProperties(kdl, exportDiagnoseDTO);
                ReflectUtil.setProperty(exportDiagnoseDTO, "type", kdl.getType().toString());
                ReflectUtil.setProperty(exportDiagnoseDTO, "orderNo", kdl.getCode());
                if(kdl.getCode().startsWith("3.")){
                    exportDiagnoseDTO.setRule(kdl.getRelation());
                }else {
                    exportDiagnoseDTO.setRule(kdl.getStandard());
                }
                exportDiagnoseDTO.setEq(kdl.getMidResult());
                eds.add(exportDiagnoseDTO);
            }
            data.put(sheetName,eds);
        }
        return data;
    }

    /**
     * 移除公式编码中没有的数据
     * @param klDiagnoseDetails
     */
    public void removeKlDiagnoseDetail(List<KlDiagnoseDetail> klDiagnoseDetails){
        List<String> regexData = getCodes(klDiagnoseDetails);
        Iterator<KlDiagnoseDetail> iterator = klDiagnoseDetails.iterator();
        while (iterator.hasNext()){
            KlDiagnoseDetail next = iterator.next();
            String code = next.getCode();
            if(!regexData.contains(code)){
                iterator.remove();
            }
        }
    }

    /**
     * 获取公式中的所有编码
     * @param klDiagnoseDetails
     * @return
     */
    private List<String> getCodes(List<KlDiagnoseDetail> klDiagnoseDetails) {
        String pateern = "([1-9]\\d*\\.?\\d*)|(0\\.\\d*[1-9])";
        List<String> collect = klDiagnoseDetails.stream().filter(x -> (x.getType().equals(91)
        || x.getType().equals(92)|| x.getType().equals(93))&& StringUtil.isNotBlank(x.getFormula()))
                .map(y -> y.getFormula()).collect(Collectors.toList());
        String content = collect.stream().collect(Collectors.joining(","));
        //获取公式中的编码
        return RegexUtil.getRegexDatas(content, pateern);
    }

}
